/*
 * Detect a Library for hardware detection
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "detect.h"
#include "utils.h"
#include "serial.h"

/* Copyrigt (c) RedHat Sotfware */
int find_legacy_mouse(int fd){
  int modem_lines;
  int temp;
  int done;
  int starttime;
  unsigned char resp[2];
  struct timeval timo;
  struct termios portattr;

  temp = get_serial_attr(fd, &portattr);
  if(temp < 0){
    return 1;
  }/*endif*/
  temp = setup_serial_port(fd, 7, &portattr);
  if(temp < 0){
    return 1;
  }/*endif*/

  modem_lines = get_serial_lines(fd);
  modem_lines &= ~(TIOCM_RTS | TIOCM_DTR);
  set_serial_lines(fd, modem_lines);
  usleep(200000);

  modem_lines |= TIOCM_DTR | TIOCM_RTS;
  set_serial_lines(fd, modem_lines);

  done = 0;
  starttime = time(NULL);
  while(!done){
    timo.tv_sec = 0;
    timo.tv_usec = 250000;
    if(wait_for_input(fd, &timo) > 0){
      temp = read(fd, resp, 1);
      if(temp < 0){
        if(errno != EAGAIN){
          return 1;
	}/*endif*/
      }else{
        done = 1;
      }/*endif*/
    }else{
      done = 1;
    }/*endif*/
    if(time(NULL) - starttime > 2){
      done = 1;
    }/*endif*/
  }/*endwhile*/

  if(*resp == 'M'){
    return 3;
  }else{
    return 2;
  }/*endif*/
}/*endfunc find_legacy_mouse*/



struct mouse_info *mouse_detect(struct bus_lst *bus){
  struct mouse_info *result = (struct mouse_info *)NULL;
  struct serial_info *serial = (struct serial_info *)NULL;
  struct usb_info *usb = (struct usb_info *)NULL;
  static struct mouse_info *first = (struct mouse_info *)NULL;
  FILE *f;
  char *line;
  char mouse_device[20] = {0};
  size_t len = 0;	
  int fd, temp, mouse_num, event_num;
  struct termios origattr;

  if(first){
    return first;
  }/*endif*/

 
  if(first){
    return first;
  }/*endif*/

  if(debug)
    fprintf(stdout, "\nProbing mouse...\n");

 /********************************************************************/
 /************************** USB MOUSE  ******************************/
 /************************** PS2 MOUSE  ******************************/
 /********************************************************************/

  if(debug){
    fprintf(stdout, "\tProbing USB and PS/2 mouse...\n");
  }/*endif*/

  if((f = fopen("/proc/bus/input/devices", "r"))){
    while(getline(&line, &len, f) >= 0) {	
      if(!strncmp(line, "N: Name=", 8)){
	if((strstr(line, "mouse")) ||
	   (strstr(line, "Mouse")) ||
	   (strstr(line, "Pad"))) { /* got mouse entry */
		if(!first){
       		 	first = result = (struct mouse_info *)
                        my_malloc(sizeof(struct mouse_info));
		} else {
		        result->next = (struct mouse_info *)
                        my_malloc(sizeof(struct mouse_info));
          	result = result->next;
        	}/*endif*/
	result->next = (struct mouse_info *)NULL;
	result->model = strdup(line + 9);
	strtok(result->model, "\"");
	result->vendor = s_unknown;
	while((getline(&line, &len, f) >= 0) && (strncmp(line, "P: Phys=", 8))); /* seek to the P entry */
	if (strstr(line, "usb")){
	  result->bus = USB;
	}
	else {
	  result->bus = PS2;
	}
	while((getline(&line, &len, f) >= 0) && (strncmp(line, "H: Handlers=", 12))); /* seek to the H entry */
	if (sscanf(line, "H: Handlers=mouse%d event%d", &mouse_num, &event_num) != 2){
	  result->device = s_not_available;
	}
	else{			/* build device string */
	  sprintf(mouse_device, "/dev/input/event%d", event_num);
	  result->device = strdup(mouse_device);
	}

		if(debug){
   		fprintf(stdout, "\t\tFound %s %s\n",
             		result->vendor, result->model);
   		}
	}
      }
    }
  len = 0;
  free(line);
  fclose(f);
 } 
  /********************************************************************/
  /******************** SERIAL MOUSE DETECTION ************************/
  /********************************************************************/
  if(debug)
    fprintf(stdout, "\tProbing serial mouse...\n");
  
  for(serial = bus->serial; serial; serial = serial->next){
    if((fd = open_serial_port(serial->device)) < 0){
      continue;
    }/*endif*/
    temp = get_serial_attr(fd, &origattr);
    if(temp < 0){
      continue;
    }/*endif*/
    temp = find_legacy_mouse(fd);
    if(temp == 3){
      if(!first){
        first = result = (struct mouse_info *) 
                                   my_malloc(sizeof(struct mouse_info));
      }else{
       	result->next = (struct mouse_info *) 
                                   my_malloc(sizeof(struct mouse_info));
        result = result->next;
      }/*endif*/
      result->next = (struct mouse_info *)NULL;
      result->vendor = s_unknown;
      result->model = s_unknown;
      result->device = serial->device;
      result->module = s_ignore;
      result->bus = SERIAL;
      if(debug)
	fprintf(stdout, "\t\tFound on %s\n", result->device);
    }/*endif*/
    set_serial_attr(fd, &origattr);
    close_serial_port(fd);
  }/*next serial*/

  return first;
}/*endfunc mouse_detect*/
